/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Tag;
import org.wso2.registry.TaggedResourcePath;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.jdbc.dataobjects.TaggingDO;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class TagsDAO {

    public void addTagging(String tagName, long resourceID, String userID, Connection conn)
            throws SQLException {

        String sql = "INSERT INTO TAGS (TAG_NAME, AID, USER_ID, TAGGED_TIME) VALUES (?,?,?,?)";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tagName);
        s.setLong(2, resourceID);
        s.setString(3, userID);
        s.setDate(4, new Date(now));

        s.executeUpdate();
    }

    public void copyTags(long fromResourceID , long toResourceID , Connection connection)
            throws SQLException{
        String SQL = "SELECT TAG_NAME , USER_ID , TAGGED_TIME FROM TAGS WHERE AID=?";
        PreparedStatement s = connection.prepareStatement(SQL);
        s.setLong(1, fromResourceID);
        ResultSet result = s.executeQuery();
        ArrayList tagList = new ArrayList();
        if (result.next()) {
            TaggingDO  taggingDO = new TaggingDO();
            taggingDO.setTagName(result.getString(1));
            taggingDO.setTaggedUserName(result.getString(2));
            taggingDO.setTaggedTime(result.getDate(3));
            tagList.add(taggingDO);
        }
        for (int i = 0; i < tagList.size(); i++) {
            TaggingDO taggingDO = (TaggingDO) tagList.get(i);
            addTagging(taggingDO.getTagName() ,toResourceID , taggingDO.getTaggedUserName() ,connection);
        }
    }

    public boolean taggingExist(String tagName, long resourceID, String userID, Connection conn)
            throws SQLException {

        String sql =
                "SELECT COUNT(TN.TAG_NAME) FROM TAGS TN WHERE TN.TAG_NAME=? AND TN.AID=? AND TN.USER_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tagName);
        s.setLong(2, resourceID);
        s.setString(3, userID);

        ResultSet result = s.executeQuery();
        long count = 0;
        if (result.next()) {
            count = result.getLong(1);
        }

        return count == 1;
    }

    public void removeTaggings(long resourceID, Connection conn) throws SQLException {

        String sql = "DELETE FROM TAGS WHERE AID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, resourceID);

        s.executeUpdate();
    }

    public void removeTag(String path, String tag, Connection conn)
            throws SQLException {

        VersionedResourceDAO resourceDAO = new VersionedResourceDAO();
        long resourceID = resourceDAO.getResourceID(path, conn);

        String sql = "DELETE FROM TAGS WHERE AID=? AND TAG_NAME=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, resourceID);
        s.setString(2, tag);

        s.executeUpdate();
    }

    public void removeTag(String path, String tag, String userName, Connection conn)
            throws SQLException {

        VersionedResourceDAO resourceDAO = new VersionedResourceDAO();
        long resourceID = resourceDAO.getResourceID(path, conn);

        String sql = "DELETE FROM TAGS WHERE AID=? AND TAG_NAME=?";
        if (!userName.equals("*")) {
            sql = sql + "AND USER_ID=?";
        }

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, resourceID);
        s.setString(2, tag);
        if (!userName.equals("*")) {
            s.setString(3, userName);
        }

        s.executeUpdate();
    }

    public String[] getPathsWithTag(String tag, Connection conn) throws SQLException {

        String sql = "SELECT A.PATH FROM ARTIFACTS A, TAGS T WHERE T.TAG_NAME=? AND T.AID=A.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        List pathList = new ArrayList();
        while (result.next()) {
            pathList.add(result.getString(DatabaseConstants.PATH_FIELD));
        }

        return (String[])pathList.toArray(new String[pathList.size()]);
    }

    public String[] getTags(String path, Connection conn) throws SQLException {

        String sql = "SELECT T.TAG_NAME FROM TAGS T, ARTIFACTS A WHERE A.PATH=? AND A.AID=T.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);

        ResultSet result = s.executeQuery();
        List tagList = new ArrayList();
        while (result.next()) {
            tagList.add(result.getString(DatabaseConstants.TAG_NAME_FIELD));
        }

        return (String[])tagList.toArray(new String[tagList.size()]);
    }

    public List getPathsWithAnyTag(String[] tags, Connection conn) throws SQLException {

        String sql = null;
        if (tags.length == 1) {
            sql = "SELECT DISTINCT R.PATH FROM ARTIFACTS R, TAGS T WHERE R.AID=T.AID AND R.STATE=" +
                  RegistryConstants.ACTIVE_STATE + " AND T.TAG_NAME=?";
        } else if (tags.length == 2) {
            sql =
                    "SELECT DISTINCT R.PATH FROM ARTIFACTS R, TAGS T WHERE R.AID=T.AID AND R.STATE=" +
                    RegistryConstants.ACTIVE_STATE + " AND (T.TAG_NAME=? OR T.TAG_NAME=?)";
        } else if (tags.length == 3) {
            sql =
                    "SELECT DISTINCT R.PATH FROM ARTIFACTS R, TAGS T WHERE R.AID=T.AID AND R.STATE=" +
                    RegistryConstants.ACTIVE_STATE +
                    " AND (T.TAG_NAME=? OR T.TAG_NAME=? OR T.TAG_NAME=?)";
        }

        PreparedStatement s = conn.prepareStatement(sql);

        for (int i = 0; i < tags.length; i++) {
            s.setString(i + 1, tags[i]);
        }

        List resourcePaths = new ArrayList();
        ResultSet results = s.executeQuery();
        while (results.next()) {
            resourcePaths.add(results.getString(DatabaseConstants.PATH_FIELD));
        }
        return resourcePaths;
    }

    public List getPathsWithAllTags(String[] tags, Connection conn) throws SQLException {
        return null;
    }

    public long getTagCount(String path, String tag, Connection conn) throws SQLException {

        String sql =
                "SELECT COUNT(TN.TAG_NAME) FROM ARTIFACTS A, TAGS TN WHERE A.PATH=? AND TN.TAG_NAME=? AND TN.AID=A.AID GROUP BY A.PATH";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);
        s.setString(2, tag);

        ResultSet result = s.executeQuery();
        if (result.next()) {
            return result.getLong(1);
        }

        return 0;
    }

    public List getTaggedPaths(String tag, Connection conn) throws SQLException {

        String sql =
                "SELECT A.PATH, COUNT(TN.TAG_NAME) FROM ARTIFACTS A, TAGS TN WHERE TN.TAG_NAME=? AND TN.AID=A.AID GROUP BY A.PATH";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        List pathList = new ArrayList();
        while (result.next()) {
            TaggedResourcePath taggedResourcePath = new TaggedResourcePath();
            taggedResourcePath.setResourcePath(result.getString(DatabaseConstants.PATH_FIELD));
            taggedResourcePath.setTagCount(result.getLong(2));
            pathList.add(taggedResourcePath);
        }

        return pathList;
    }

    public Tag[] getTagsWithCount(String path, Connection conn) throws SQLException {

        String sql =
                "SELECT TN.TAG_NAME, COUNT(A.AID) FROM TAGS TN, ARTIFACTS A WHERE A.PATH=? AND A.AID=TN.AID GROUP BY TN.TAG_NAME";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);

        ResultSet result = s.executeQuery();
        List tagList = new ArrayList();
        while (result.next()) {
            Tag tag = new Tag();
            tag.setTagName(result.getString(DatabaseConstants.TAG_NAME_FIELD));
            tag.setTagCount(result.getLong(2));
            tagList.add(tag);
        }

        return (Tag[])tagList.toArray(new Tag[tagList.size()]);
    }

    public TaggingDO getTagging(String path, String tagName, String userName, Connection conn)
            throws SQLException {

        VersionedResourceDAO resourceDAO = new VersionedResourceDAO();
        long aid = resourceDAO.getResourceID(path, conn);

        String sql =
                "SELECT T.TAGGED_TIME FROM TAGS T, ARTIFACTS A WHERE A.AID=T.AID AND T.AID=? AND T.USER_ID=? AND T.TAG_NAME=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, aid);
        s.setString(2, userName);
        s.setString(3, tagName);

        ResultSet result = s.executeQuery();
        if (result.next()) {

            java.util.Date taggedTime = new java.util.Date(
                    result.getTimestamp(DatabaseConstants.TAGGED_TIME_FIELD).getTime());

            TaggingDO taggingDO = new TaggingDO();
            taggingDO.setResourceID(aid);
            taggingDO.setResourcePath(path);
            taggingDO.setTagName(tagName);
            taggingDO.setTaggedTime(taggedTime);
            taggingDO.setTaggedUserName(userName);

            return taggingDO;
        }

        return null;
    }

    public TaggingDO getTagging(long taggingID, Connection conn)
            throws SQLException {

        String sql =
                "SELECT A.AID, A.PATH, T.USER_ID, T.TAG_NAME, T.TAGGED_TIME FROM TAGS T, ARTIFACTS A WHERE A.AID=T.AID AND T.TN_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, taggingID);

        ResultSet result = s.executeQuery();
        if (result.next()) {

            java.util.Date taggedTime = new java.util.Date(
                    result.getTimestamp(DatabaseConstants.TAGGED_TIME_FIELD).getTime());

            TaggingDO taggingDO = new TaggingDO();
            taggingDO.setResourceID(result.getLong(DatabaseConstants.AID_FIELD));
            taggingDO.setResourcePath(result.getString(DatabaseConstants.PATH_FIELD));
            taggingDO.setTagName(result.getString(DatabaseConstants.TAG_NAME_FIELD));
            taggingDO.setTaggedUserName(result.getString(DatabaseConstants.USER_ID_FIELD));
            taggingDO.setTaggedTime(taggedTime);

            return taggingDO;
        }

        return null;
    }
}
